@@ -30,6 +30,7 @@ Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 #include "command.h"
 #include "thread.h"
 #include "smux.h"
+#include "filter.h"
 
 #include "bgpd/bgpd.h"
 #include "bgpd/bgp_table.h"
@@ -74,9 +75,8 @@ Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 #define BGPPEERKEEPALIVE                     19
 #define BGPPEERHOLDTIMECONFIGURED            20
 #define BGPPEERKEEPALIVECONFIGURED           21
-#define BGPPEERMINASORIGINATIONINTERVAL      22
-#define BGPPEERMINROUTEADVERTISEMENTINTERVAL 23
-#define BGPPEERINUPDATEELAPSEDTIME           24
+#define BGPPEERMINROUTEADVERTISEMENTINTERVAL 22
+#define BGPPEERINUPDATEELAPSEDTIME           23
 
 /* BGP MIB bgpIdentifier. */
 #define BGPIDENTIFIER                         0
@@ -121,7 +121,7 @@ oid bgp_oid [] = { BGP4MIB };
 oid bgp_trap_oid [] = { BGP4MIB, 0 };
 
 /* IP address 0.0.0.0. */
-static struct in_addr bgp_empty_addr = {0};
+static struct in_addr bgp_empty_addr = { .s_addr = 0 };
 
 /* Hook functions. */
 static u_char *bgpVersion (struct variable *, oid [], size_t *, int,
@@ -189,8 +189,6 @@ struct variable bgp_variables[] =
    3, {3, 1, 20}},
   {BGPPEERKEEPALIVECONFIGURED, INTEGER, RWRITE, bgpPeerTable,
    3, {3, 1, 21}},
-  {BGPPEERMINASORIGINATIONINTERVAL, INTEGER, RWRITE, bgpPeerTable,
-   3, {3, 1, 22}},
   {BGPPEERMINROUTEADVERTISEMENTINTERVAL, INTEGER, RWRITE, bgpPeerTable,
    3, {3, 1, 23}},
   {BGPPEERINUPDATEELAPSEDTIME, GAUGE32, RONLY, bgpPeerTable,
@@ -337,38 +335,42 @@ bgp_peer_lookup_next (struct in_addr *src)
   return NULL;
 }
 
+/* 1.3.6.1.2.1.15.3.1.x  = 10 */
+#define PEERTAB_NAMELEN 10
+
 static struct peer *
 bgpPeerTable_lookup (struct variable *v, oid name[], size_t *length, 
 		     struct in_addr *addr, int exact)
 {
   struct peer *peer = NULL;
+  size_t namelen = v ? v->namelen : PEERTAB_NAMELEN;
   int len;
 
   if (exact)
     {
       /* Check the length. */
-      if (*length - v->namelen != sizeof (struct in_addr))
+      if (*length - namelen != sizeof (struct in_addr))
 	return NULL;
 
-      oid2in_addr (name + v->namelen, IN_ADDR_SIZE, addr);
+      oid2in_addr (name + namelen, IN_ADDR_SIZE, addr);
 
       peer = peer_lookup_addr_ipv4 (addr);
       return peer;
     }
   else
     {
-      len = *length - v->namelen;
+      len = *length - namelen;
       if (len > 4) len = 4;
       
-      oid2in_addr (name + v->namelen, len, addr);
+      oid2in_addr (name + namelen, len, addr);
       
       peer = bgp_peer_lookup_next (addr);
 
       if (peer == NULL)
 	return NULL;
 
-      oid_copy_addr (name + v->namelen, addr, sizeof (struct in_addr));
-      *length = sizeof (struct in_addr) + v->namelen;
+      oid_copy_addr (name + namelen, addr, sizeof (struct in_addr));
+      *length = sizeof (struct in_addr) + namelen;
 
       return peer;
     }
@@ -379,14 +381,12 @@ bgpPeerTable_lookup (struct variable *v, oid name[], size_t *length,
 static int
 write_bgpPeerTable (int action, u_char *var_val,
 		    u_char var_val_type, size_t var_val_len,
-		    u_char *statP, oid *name, size_t length,
-		    struct variable *v)
+		    u_char *statP, oid *name, size_t length)
 {
   struct in_addr addr;
   struct peer *peer;
   long intval;
-  size_t bigsize = SNMP_MAX_LEN;
-  
+
   if (var_val_type != ASN_INTEGER) 
     {
       return SNMP_ERR_WRONGTYPE;
@@ -396,21 +396,21 @@ write_bgpPeerTable (int action, u_char *var_val,
       return SNMP_ERR_WRONGLENGTH;
     }
 
-  if (! asn_parse_int(var_val, &bigsize, &var_val_type,
-                      &intval, sizeof(long)))
-    {
-      return SNMP_ERR_WRONGENCODING;
-    }
+  intval = *(long *)var_val;
 
   memset (&addr, 0, sizeof (struct in_addr));
 
-  peer = bgpPeerTable_lookup (v, name, &length, &addr, 1);
+  peer = bgpPeerTable_lookup (NULL, name, &length, &addr, 1);
   if (! peer)
     return SNMP_ERR_NOSUCHNAME;
 
-  printf ("val: %ld\n", intval);
+  if (action != SNMP_MSG_INTERNAL_SET_COMMIT)
+    return SNMP_ERR_NOERROR;
 
-  switch (v->magic)
+  zlog_info ("%s: SNMP write .%ld = %ld",
+             peer->host, (long)name[PEERTAB_NAMELEN - 1], intval);
+
+  switch (name[PEERTAB_NAMELEN - 1])
     {
     case BGPPEERADMINSTATUS:
 #define BGP_PeerAdmin_stop  1
@@ -438,9 +438,6 @@ write_bgpPeerTable (int action, u_char *var_val,
       peer->keepalive = intval;
       peer->v_keepalive = intval;
       break;
-    case BGPPEERMINASORIGINATIONINTERVAL:
-      peer->v_asorig = intval;
-      break;
     case BGPPEERMINROUTEADVERTISEMENTINTERVAL:
       peer->v_routeadv = intval;
       break;
@@ -569,10 +566,6 @@ bgpPeerTable (struct variable *v, oid name[], size_t *length,
       else
 	return SNMP_INTEGER (peer->v_keepalive);
       break;
-    case BGPPEERMINASORIGINATIONINTERVAL:
-      *write_method = write_bgpPeerTable;
-      return SNMP_INTEGER (peer->v_asorig);
-      break;
     case BGPPEERMINROUTEADVERTISEMENTINTERVAL:
       *write_method = write_bgpPeerTable;
       return SNMP_INTEGER (peer->v_routeadv);
