@@ -22,11 +22,6 @@
 #ifndef _ZEBRA_ISIS_EVENTS_H
 #define _ZEBRA_ISIS_EVENTS_H
 
-/*
- * Events related to area
- */
-void isis_event_system_type_change (struct isis_area *area, int newtype);
-
 /*
  * Events related to circuit
  */
