@@ -1033,7 +1033,7 @@ isis_spf_preload_tent (struct isis_spftree *spftree, int level,
 		  break;
 		case ISIS_SYSTYPE_UNKNOWN:
 		default:
-		  zlog_warn ("isis_spf_preload_tent unknow adj type");
+		  zlog_warn ("isis_spf_preload_tent unknown adj type");
 		}
 	    }
 	  list_delete (adj_list);
