@@ -45,22 +45,33 @@ distribute_new (void)
 static void
 distribute_free (struct distribute *dist)
 {
+  int i = 0;
   if (dist->ifname)
     XFREE (MTYPE_DISTRIBUTE_IFNAME, dist->ifname);
 
-  if (dist->list[DISTRIBUTE_IN])
-    free (dist->list[DISTRIBUTE_IN]);
-  if (dist->list[DISTRIBUTE_OUT])
-    free (dist->list[DISTRIBUTE_OUT]);
+  for (i=0; i < DISTRIBUTE_MAX; i++)
+    if (dist->list[i])
+      free(dist->list[i]);
 
-  if (dist->prefix[DISTRIBUTE_IN])
-    free (dist->prefix[DISTRIBUTE_IN]);
-  if (dist->prefix[DISTRIBUTE_OUT])
-    free (dist->prefix[DISTRIBUTE_OUT]);
+  for (i=0; i < DISTRIBUTE_MAX; i++)
+    if (dist->prefix[i])
+      free(dist->prefix[i]);
 
   XFREE (MTYPE_DISTRIBUTE, dist);
 }
 
+static void
+distribute_free_if_empty(struct distribute *dist)
+{
+  int i;
+  for (i=0; i < DISTRIBUTE_MAX; i++)
+    if (dist->list[i] != NULL || dist->prefix[i] != NULL)
+      return;
+
+  hash_release (disthash, dist);
+  distribute_free (dist);
+}
+
 /* Lookup interface's distribute list. */
 struct distribute *
 distribute_lookup (const char *ifname)
@@ -136,36 +147,27 @@ distribute_cmp (const struct distribute *dist1, const struct distribute *dist2)
 
 /* Set access-list name to the distribute list. */
 static struct distribute *
-distribute_list_set (const char *ifname, enum distribute_type type, 
+distribute_list_set (const char *ifname, enum distribute_type type,
                      const char *alist_name)
 {
   struct distribute *dist;
 
   dist = distribute_get (ifname);
 
-  if (type == DISTRIBUTE_IN)
-    {
-      if (dist->list[DISTRIBUTE_IN])
-	free (dist->list[DISTRIBUTE_IN]);
-      dist->list[DISTRIBUTE_IN] = strdup (alist_name);
-    }
-  if (type == DISTRIBUTE_OUT)
-    {
-      if (dist->list[DISTRIBUTE_OUT])
-	free (dist->list[DISTRIBUTE_OUT]);
-      dist->list[DISTRIBUTE_OUT] = strdup (alist_name);
-    }
+  if (dist->list[type])
+    free (dist->list[type]);
+  dist->list[type] = strdup (alist_name);
 
   /* Apply this distribute-list to the interface. */
   (*distribute_add_hook) (dist);
-  
+
   return dist;
 }
 
 /* Unset distribute-list.  If matched distribute-list exist then
    return 1. */
 static int
-distribute_list_unset (const char *ifname, enum distribute_type type, 
+distribute_list_unset (const char *ifname, enum distribute_type type,
 		       const char *alist_name)
 {
   struct distribute *dist;
@@ -174,41 +176,19 @@ distribute_list_unset (const char *ifname, enum distribute_type type,
   if (!dist)
     return 0;
 
-  if (type == DISTRIBUTE_IN)
-    {
-      if (!dist->list[DISTRIBUTE_IN])
-	return 0;
-      if (strcmp (dist->list[DISTRIBUTE_IN], alist_name) != 0)
-	return 0;
+  if (!dist->list[type])
+    return 0;
+  if (strcmp (dist->list[type], alist_name) != 0)
+    return 0;
 
-      free (dist->list[DISTRIBUTE_IN]);
-      dist->list[DISTRIBUTE_IN] = NULL;      
-    }
-
-  if (type == DISTRIBUTE_OUT)
-    {
-      if (!dist->list[DISTRIBUTE_OUT])
-	return 0;
-      if (strcmp (dist->list[DISTRIBUTE_OUT], alist_name) != 0)
-	return 0;
-
-      free (dist->list[DISTRIBUTE_OUT]);
-      dist->list[DISTRIBUTE_OUT] = NULL;      
-    }
+  free (dist->list[type]);
+  dist->list[type] = NULL;
 
   /* Apply this distribute-list to the interface. */
   (*distribute_delete_hook) (dist);
 
-  /* If both out and in is NULL then free distribute list. */
-  if (dist->list[DISTRIBUTE_IN] == NULL &&
-      dist->list[DISTRIBUTE_OUT] == NULL &&
-      dist->prefix[DISTRIBUTE_IN] == NULL &&
-      dist->prefix[DISTRIBUTE_OUT] == NULL)
-    {
-      hash_release (disthash, dist);
-      distribute_free (dist);
-    }
-
+  /* If all dist are NULL, then free distribute list. */
+  distribute_free_if_empty(dist);
   return 1;
 }
 
@@ -221,18 +201,9 @@ distribute_list_prefix_set (const char *ifname, enum distribute_type type,
 
   dist = distribute_get (ifname);
 
-  if (type == DISTRIBUTE_IN)
-    {
-      if (dist->prefix[DISTRIBUTE_IN])
-	free (dist->prefix[DISTRIBUTE_IN]);
-      dist->prefix[DISTRIBUTE_IN] = strdup (plist_name);
-    }
-  if (type == DISTRIBUTE_OUT)
-    {
-      if (dist->prefix[DISTRIBUTE_OUT])
-	free (dist->prefix[DISTRIBUTE_OUT]);
-      dist->prefix[DISTRIBUTE_OUT] = strdup (plist_name);
-    }
+  if (dist->prefix[type])
+    free (dist->prefix[type]);
+  dist->prefix[type] = strdup (plist_name);
 
   /* Apply this distribute-list to the interface. */
   (*distribute_add_hook) (dist);
@@ -252,41 +223,19 @@ distribute_list_prefix_unset (const char *ifname, enum distribute_type type,
   if (!dist)
     return 0;
 
-  if (type == DISTRIBUTE_IN)
-    {
-      if (!dist->prefix[DISTRIBUTE_IN])
-	return 0;
-      if (strcmp (dist->prefix[DISTRIBUTE_IN], plist_name) != 0)
-	return 0;
+  if (!dist->prefix[type])
+    return 0;
+  if (strcmp (dist->prefix[type], plist_name) != 0)
+    return 0;
 
-      free (dist->prefix[DISTRIBUTE_IN]);
-      dist->prefix[DISTRIBUTE_IN] = NULL;      
-    }
-
-  if (type == DISTRIBUTE_OUT)
-    {
-      if (!dist->prefix[DISTRIBUTE_OUT])
-	return 0;
-      if (strcmp (dist->prefix[DISTRIBUTE_OUT], plist_name) != 0)
-	return 0;
-
-      free (dist->prefix[DISTRIBUTE_OUT]);
-      dist->prefix[DISTRIBUTE_OUT] = NULL;      
-    }
+  free (dist->prefix[type]);
+  dist->prefix[type] = NULL;
 
   /* Apply this distribute-list to the interface. */
   (*distribute_delete_hook) (dist);
 
-  /* If both out and in is NULL then free distribute list. */
-  if (dist->list[DISTRIBUTE_IN] == NULL &&
-      dist->list[DISTRIBUTE_OUT] == NULL &&
-      dist->prefix[DISTRIBUTE_IN] == NULL &&
-      dist->prefix[DISTRIBUTE_OUT] == NULL)
-    {
-      hash_release (disthash, dist);
-      distribute_free (dist);
-    }
-
+  /* If all dist are NULL, then free distribute list. */
+  distribute_free_if_empty(dist);
   return 1;
 }
 
@@ -302,9 +251,9 @@ DEFUN (distribute_list_all,
 
   /* Check of distribute list type. */
   if (strncmp (argv[1], "i", 1) == 0)
-    type = DISTRIBUTE_IN;
+    type = DISTRIBUTE_V4_IN;
   else if (strncmp (argv[1], "o", 1) == 0)
-    type = DISTRIBUTE_OUT;
+    type = DISTRIBUTE_V4_OUT;
   else
     {
       vty_out (vty, "distribute list direction must be [in|out]%s",
@@ -318,8 +267,36 @@ DEFUN (distribute_list_all,
   return CMD_SUCCESS;
 }
 
-ALIAS (distribute_list_all,
+DEFUN (ipv6_distribute_list_all,
        ipv6_distribute_list_all_cmd,
+       "ipv6 distribute-list WORD (in|out)",
+       "Filter networks in routing updates\n"
+       "Access-list name\n"
+       "Filter incoming routing updates\n"
+       "Filter outgoing routing updates\n")
+{
+  enum distribute_type type;
+
+  /* Check of distribute list type. */
+  if (strncmp (argv[1], "i", 1) == 0)
+    type = DISTRIBUTE_V6_IN;
+  else if (strncmp (argv[1], "o", 1) == 0)
+    type = DISTRIBUTE_V6_OUT;
+  else
+  {
+    vty_out (vty, "distribute list direction must be [in|out]%s",
+             VTY_NEWLINE);
+    return CMD_WARNING;
+  }
+
+  /* Get interface name corresponding distribute list. */
+  distribute_list_set (NULL, type, argv[0]);
+
+  return CMD_SUCCESS;
+}
+
+ALIAS (ipv6_distribute_list_all,
+       ipv6_as_v4_distribute_list_all_cmd,
        "distribute-list WORD (in|out)",
        "Filter networks in routing updates\n"
        "Access-list name\n"
@@ -340,9 +317,9 @@ DEFUN (no_distribute_list_all,
 
   /* Check of distribute list type. */
   if (strncmp (argv[1], "i", 1) == 0)
-    type = DISTRIBUTE_IN;
+    type = DISTRIBUTE_V4_IN;
   else if (strncmp (argv[1], "o", 1) == 0)
-    type = DISTRIBUTE_OUT;
+    type = DISTRIBUTE_V4_OUT;
   else
     {
       vty_out (vty, "distribute list direction must be [in|out]%s",
@@ -359,8 +336,41 @@ DEFUN (no_distribute_list_all,
   return CMD_SUCCESS;
 }
 
-ALIAS (no_distribute_list_all,
+DEFUN (no_ipv6_distribute_list_all,
        no_ipv6_distribute_list_all_cmd,
+       "no ipv6 distribute-list WORD (in|out)",
+       NO_STR
+       "Filter networks in routing updates\n"
+       "Access-list name\n"
+       "Filter incoming routing updates\n"
+       "Filter outgoing routing updates\n")
+{
+  int ret;
+  enum distribute_type type;
+
+  /* Check of distribute list type. */
+  if (strncmp (argv[1], "i", 1) == 0)
+    type = DISTRIBUTE_V6_IN;
+  else if (strncmp (argv[1], "o", 1) == 0)
+    type = DISTRIBUTE_V6_OUT;
+  else
+  {
+    vty_out (vty, "distribute list direction must be [in|out]%s",
+             VTY_NEWLINE);
+    return CMD_WARNING;
+  }
+
+  ret = distribute_list_unset (NULL, type, argv[0]);
+  if (! ret)
+  {
+    vty_out (vty, "distribute list doesn't exist%s", VTY_NEWLINE);
+    return CMD_WARNING;
+  }
+  return CMD_SUCCESS;
+}
+
+ALIAS (no_ipv6_distribute_list_all,
+       no_ipv6_as_v4_distribute_list_all_cmd,
        "no distribute-list WORD (in|out)",
        NO_STR
        "Filter networks in routing updates\n"
@@ -381,9 +391,9 @@ DEFUN (distribute_list,
 
   /* Check of distribute list type. */
   if (strncmp (argv[1], "i", 1) == 0)
-    type = DISTRIBUTE_IN;
+    type = DISTRIBUTE_V4_IN;
   else if (strncmp (argv[1], "o", 1) == 0)
-    type = DISTRIBUTE_OUT;
+    type = DISTRIBUTE_V4_OUT;
   else
     {
       vty_out (vty, "distribute list direction must be [in|out]%s", VTY_NEWLINE);
@@ -394,10 +404,38 @@ DEFUN (distribute_list,
   distribute_list_set (argv[2], type, argv[0]);
 
   return CMD_SUCCESS;
-}       
+}
 
-ALIAS (distribute_list,
+DEFUN (ipv6_distribute_list,
        ipv6_distribute_list_cmd,
+       "ipv6 distribute-list WORD (in|out) WORD",
+       "Filter networks in routing updates\n"
+       "Access-list name\n"
+       "Filter incoming routing updates\n"
+       "Filter outgoing routing updates\n"
+       "Interface name\n")
+{
+  enum distribute_type type;
+
+  /* Check of distribute list type. */
+  if (strncmp (argv[1], "i", 1) == 0)
+    type = DISTRIBUTE_V6_IN;
+  else if (strncmp (argv[1], "o", 1) == 0)
+    type = DISTRIBUTE_V6_OUT;
+  else
+  {
+    vty_out (vty, "distribute list direction must be [in|out]%s", VTY_NEWLINE);
+    return CMD_WARNING;
+  }
+
+  /* Get interface name corresponding distribute list. */
+  distribute_list_set (argv[2], type, argv[0]);
+
+  return CMD_SUCCESS;
+}
+
+ALIAS (ipv6_distribute_list,
+       ipv6_as_v4_distribute_list_cmd,
        "distribute-list WORD (in|out) WORD",
        "Filter networks in routing updates\n"
        "Access-list name\n"
@@ -419,9 +457,9 @@ DEFUN (no_distribute_list, no_distribute_list_cmd,
 
   /* Check of distribute list type. */
   if (strncmp (argv[1], "i", 1) == 0)
-    type = DISTRIBUTE_IN;
+    type = DISTRIBUTE_V4_IN;
   else if (strncmp (argv[1], "o", 1) == 0)
-    type = DISTRIBUTE_OUT;
+    type = DISTRIBUTE_V4_OUT;
   else
     {
       vty_out (vty, "distribute list direction must be [in|out]%s", VTY_NEWLINE);
@@ -435,9 +473,43 @@ DEFUN (no_distribute_list, no_distribute_list_cmd,
       return CMD_WARNING;
     }
   return CMD_SUCCESS;
-}       
+}
 
-ALIAS (no_distribute_list, no_ipv6_distribute_list_cmd,
+DEFUN (no_ipv6_distribute_list,
+       no_ipv6_distribute_list_cmd,
+       "no ipv6 distribute-list WORD (in|out) WORD",
+       NO_STR
+       "Filter networks in routing updates\n"
+       "Access-list name\n"
+       "Filter incoming routing updates\n"
+       "Filter outgoing routing updates\n"
+       "Interface name\n")
+{
+  int ret;
+  enum distribute_type type;
+
+  /* Check of distribute list type. */
+  if (strncmp (argv[1], "i", 1) == 0)
+    type = DISTRIBUTE_V6_IN;
+  else if (strncmp (argv[1], "o", 1) == 0)
+    type = DISTRIBUTE_V6_OUT;
+  else
+  {
+    vty_out (vty, "distribute list direction must be [in|out]%s", VTY_NEWLINE);
+    return CMD_WARNING;
+  }
+
+  ret = distribute_list_unset (argv[2], type, argv[0]);
+  if (! ret)
+  {
+    vty_out (vty, "distribute list doesn't exist%s", VTY_NEWLINE);
+    return CMD_WARNING;
+  }
+  return CMD_SUCCESS;
+}
+
+ALIAS (no_ipv6_distribute_list,
+       no_ipv6_as_v4_distribute_list_cmd,
        "no distribute-list WORD (in|out) WORD",
        NO_STR
        "Filter networks in routing updates\n"
@@ -459,12 +531,12 @@ DEFUN (distribute_list_prefix_all,
 
   /* Check of distribute list type. */
   if (strncmp (argv[1], "i", 1) == 0)
-    type = DISTRIBUTE_IN;
+    type = DISTRIBUTE_V4_IN;
   else if (strncmp (argv[1], "o", 1) == 0)
-    type = DISTRIBUTE_OUT;
+    type = DISTRIBUTE_V4_OUT;
   else
     {
-      vty_out (vty, "distribute list direction must be [in|out]%s", 
+      vty_out (vty, "distribute list direction must be [in|out]%s",
 	       VTY_NEWLINE);
       return CMD_WARNING;
     }
@@ -473,10 +545,39 @@ DEFUN (distribute_list_prefix_all,
   distribute_list_prefix_set (NULL, type, argv[0]);
 
   return CMD_SUCCESS;
-}       
+}
 
-ALIAS (distribute_list_prefix_all,
+DEFUN (ipv6_distribute_list_prefix_all,
        ipv6_distribute_list_prefix_all_cmd,
+       "ipv6 distribute-list prefix WORD (in|out)",
+       "Filter networks in routing updates\n"
+       "Filter prefixes in routing updates\n"
+       "Name of an IP prefix-list\n"
+       "Filter incoming routing updates\n"
+       "Filter outgoing routing updates\n")
+{
+  enum distribute_type type;
+
+  /* Check of distribute list type. */
+  if (strncmp (argv[1], "i", 1) == 0)
+    type = DISTRIBUTE_V6_IN;
+  else if (strncmp (argv[1], "o", 1) == 0)
+    type = DISTRIBUTE_V6_OUT;
+  else
+  {
+    vty_out (vty, "distribute list direction must be [in|out]%s",
+             VTY_NEWLINE);
+    return CMD_WARNING;
+  }
+
+  /* Get interface name corresponding distribute list. */
+  distribute_list_prefix_set (NULL, type, argv[0]);
+
+  return CMD_SUCCESS;
+}
+
+ALIAS (ipv6_distribute_list_prefix_all,
+       ipv6_as_v4_distribute_list_prefix_all_cmd,
        "distribute-list prefix WORD (in|out)",
        "Filter networks in routing updates\n"
        "Filter prefixes in routing updates\n"
@@ -499,12 +600,12 @@ DEFUN (no_distribute_list_prefix_all,
 
   /* Check of distribute list type. */
   if (strncmp (argv[1], "i", 1) == 0)
-    type = DISTRIBUTE_IN;
+    type = DISTRIBUTE_V4_IN;
   else if (strncmp (argv[1], "o", 1) == 0)
-    type = DISTRIBUTE_OUT;
+    type = DISTRIBUTE_V4_OUT;
   else
     {
-      vty_out (vty, "distribute list direction must be [in|out]%s", 
+      vty_out (vty, "distribute list direction must be [in|out]%s",
 	       VTY_NEWLINE);
       return CMD_WARNING;
     }
@@ -516,10 +617,44 @@ DEFUN (no_distribute_list_prefix_all,
       return CMD_WARNING;
     }
   return CMD_SUCCESS;
-}       
+}
 
-ALIAS (no_distribute_list_prefix_all,
+DEFUN (no_ipv6_distribute_list_prefix_all,
        no_ipv6_distribute_list_prefix_all_cmd,
+       "no ipv6 distribute-list prefix WORD (in|out)",
+       NO_STR
+       "Filter networks in routing updates\n"
+       "Filter prefixes in routing updates\n"
+       "Name of an IP prefix-list\n"
+       "Filter incoming routing updates\n"
+       "Filter outgoing routing updates\n")
+{
+  int ret;
+  enum distribute_type type;
+
+  /* Check of distribute list type. */
+  if (strncmp (argv[1], "i", 1) == 0)
+    type = DISTRIBUTE_V6_IN;
+  else if (strncmp (argv[1], "o", 1) == 0)
+    type = DISTRIBUTE_V6_OUT;
+  else
+  {
+    vty_out (vty, "distribute list direction must be [in|out]%s",
+             VTY_NEWLINE);
+    return CMD_WARNING;
+  }
+
+  ret = distribute_list_prefix_unset (NULL, type, argv[0]);
+  if (! ret)
+  {
+    vty_out (vty, "distribute list doesn't exist%s", VTY_NEWLINE);
+    return CMD_WARNING;
+  }
+  return CMD_SUCCESS;
+}
+
+ALIAS (no_ipv6_distribute_list_prefix_all,
+       no_ipv6_as_v4_distribute_list_prefix_all_cmd,
        "no distribute-list prefix WORD (in|out)",
        NO_STR
        "Filter networks in routing updates\n"
@@ -541,12 +676,12 @@ DEFUN (distribute_list_prefix, distribute_list_prefix_cmd,
 
   /* Check of distribute list type. */
   if (strncmp (argv[1], "i", 1) == 0)
-    type = DISTRIBUTE_IN;
+    type = DISTRIBUTE_V4_IN;
   else if (strncmp (argv[1], "o", 1) == 0)
-    type = DISTRIBUTE_OUT;
+    type = DISTRIBUTE_V4_OUT;
   else
     {
-      vty_out (vty, "distribute list direction must be [in|out]%s", 
+      vty_out (vty, "distribute list direction must be [in|out]%s",
 	       VTY_NEWLINE);
       return CMD_WARNING;
     }
@@ -555,9 +690,40 @@ DEFUN (distribute_list_prefix, distribute_list_prefix_cmd,
   distribute_list_prefix_set (argv[2], type, argv[0]);
 
   return CMD_SUCCESS;
-}       
+}
 
-ALIAS (distribute_list_prefix, ipv6_distribute_list_prefix_cmd,
+DEFUN (ipv6_distribute_list_prefix,
+       ipv6_distribute_list_prefix_cmd,
+       "ipv6 distribute-list prefix WORD (in|out) WORD",
+       "Filter networks in routing updates\n"
+       "Filter prefixes in routing updates\n"
+       "Name of an IP prefix-list\n"
+       "Filter incoming routing updates\n"
+       "Filter outgoing routing updates\n"
+       "Interface name\n")
+{
+  enum distribute_type type;
+
+  /* Check of distribute list type. */
+  if (strncmp (argv[1], "i", 1) == 0)
+    type = DISTRIBUTE_V6_IN;
+  else if (strncmp (argv[1], "o", 1) == 0)
+    type = DISTRIBUTE_V6_OUT;
+  else
+  {
+    vty_out (vty, "distribute list direction must be [in|out]%s",
+             VTY_NEWLINE);
+    return CMD_WARNING;
+  }
+
+  /* Get interface name corresponding distribute list. */
+  distribute_list_prefix_set (argv[2], type, argv[0]);
+
+  return CMD_SUCCESS;
+}
+
+ALIAS (ipv6_distribute_list_prefix,
+       ipv6_as_v4_distribute_list_prefix_cmd,
        "distribute-list prefix WORD (in|out) WORD",
        "Filter networks in routing updates\n"
        "Filter prefixes in routing updates\n"
@@ -581,12 +747,12 @@ DEFUN (no_distribute_list_prefix, no_distribute_list_prefix_cmd,
 
   /* Check of distribute list type. */
   if (strncmp (argv[1], "i", 1) == 0)
-    type = DISTRIBUTE_IN;
+    type = DISTRIBUTE_V4_IN;
   else if (strncmp (argv[1], "o", 1) == 0)
-    type = DISTRIBUTE_OUT;
+    type = DISTRIBUTE_V4_OUT;
   else
     {
-      vty_out (vty, "distribute list direction must be [in|out]%s", 
+      vty_out (vty, "distribute list direction must be [in|out]%s",
 	       VTY_NEWLINE);
       return CMD_WARNING;
     }
@@ -598,9 +764,45 @@ DEFUN (no_distribute_list_prefix, no_distribute_list_prefix_cmd,
       return CMD_WARNING;
     }
   return CMD_SUCCESS;
-}       
+}
 
-ALIAS (no_distribute_list_prefix, no_ipv6_distribute_list_prefix_cmd,
+DEFUN (no_ipv6_distribute_list_prefix,
+       no_ipv6_distribute_list_prefix_cmd,
+       "no ipv6 distribute-list prefix WORD (in|out) WORD",
+       NO_STR
+       "Filter networks in routing updates\n"
+       "Filter prefixes in routing updates\n"
+       "Name of an IP prefix-list\n"
+       "Filter incoming routing updates\n"
+       "Filter outgoing routing updates\n"
+       "Interface name\n")
+{
+  int ret;
+  enum distribute_type type;
+
+  /* Check of distribute list type. */
+  if (strncmp (argv[1], "i", 1) == 0)
+    type = DISTRIBUTE_V6_IN;
+  else if (strncmp (argv[1], "o", 1) == 0)
+    type = DISTRIBUTE_V6_OUT;
+  else
+  {
+    vty_out (vty, "distribute list direction must be [in|out]%s",
+             VTY_NEWLINE);
+    return CMD_WARNING;
+  }
+
+  ret = distribute_list_prefix_unset (argv[2], type, argv[0]);
+  if (! ret)
+  {
+    vty_out (vty, "distribute list doesn't exist%s", VTY_NEWLINE);
+    return CMD_WARNING;
+  }
+  return CMD_SUCCESS;
+}
+
+ALIAS (no_ipv6_distribute_list_prefix,
+       no_ipv6_as_v4_distribute_list_prefix_cmd,
        "no distribute-list prefix WORD (in|out) WORD",
        NO_STR
        "Filter networks in routing updates\n"
@@ -610,80 +812,113 @@ ALIAS (no_distribute_list_prefix, no_ipv6_distribute_list_prefix_cmd,
        "Filter outgoing routing updates\n"
        "Interface name\n")
 
+static int
+distribute_print (struct vty *vty, char *tab[], int is_prefix,
+                  enum distribute_type type, int has_print)
+{
+  if (tab[type]) {
+    vty_out (vty, "%s %s%s",
+             has_print ? "," : "",
+             is_prefix ? "(prefix-list) " : "",
+             tab[type]);
+    return 1;
+  }
+  return has_print;
+}
+
 int
 config_show_distribute (struct vty *vty)
 {
   unsigned int i;
+  int has_print = 0;
   struct hash_backet *mp;
   struct distribute *dist;
 
   /* Output filter configuration. */
   dist = distribute_lookup (NULL);
-  if (dist && (dist->list[DISTRIBUTE_OUT] || dist->prefix[DISTRIBUTE_OUT]))
+  vty_out(vty, "  Outgoing update filter list for all interface is");
+  has_print = 0;
+  if (dist)
     {
-      vty_out (vty, "  Outgoing update filter list for all interface is");
-      if (dist->list[DISTRIBUTE_OUT])
-	vty_out (vty, " %s", dist->list[DISTRIBUTE_OUT]);
-      if (dist->prefix[DISTRIBUTE_OUT])
-	vty_out (vty, "%s (prefix-list) %s",
-		 dist->list[DISTRIBUTE_OUT] ? "," : "",
-		 dist->prefix[DISTRIBUTE_OUT]);
-      vty_out (vty, "%s", VTY_NEWLINE);
+      has_print = distribute_print(vty, dist->list,   0,
+                                   DISTRIBUTE_V4_OUT, has_print);
+      has_print = distribute_print(vty, dist->prefix, 1,
+                                   DISTRIBUTE_V4_OUT, has_print);
+      has_print = distribute_print(vty, dist->list,   0,
+                                   DISTRIBUTE_V6_OUT, has_print);
+      has_print = distribute_print(vty, dist->prefix, 1,
+                                   DISTRIBUTE_V6_OUT, has_print);
     }
+  if (has_print)
+    vty_out (vty, "%s", VTY_NEWLINE);
   else
-    vty_out (vty, "  Outgoing update filter list for all interface is not set%s", VTY_NEWLINE);
+    vty_out (vty, " not set%s", VTY_NEWLINE);
 
   for (i = 0; i < disthash->size; i++)
     for (mp = disthash->index[i]; mp; mp = mp->next)
       {
 	dist = mp->data;
 	if (dist->ifname)
-	  if (dist->list[DISTRIBUTE_OUT] || dist->prefix[DISTRIBUTE_OUT])
-	    {
-	      vty_out (vty, "    %s filtered by", dist->ifname);
-	      if (dist->list[DISTRIBUTE_OUT])
-		vty_out (vty, " %s", dist->list[DISTRIBUTE_OUT]);
-	      if (dist->prefix[DISTRIBUTE_OUT])
-		vty_out (vty, "%s (prefix-list) %s",
-			 dist->list[DISTRIBUTE_OUT] ? "," : "",
-			 dist->prefix[DISTRIBUTE_OUT]);
-	      vty_out (vty, "%s", VTY_NEWLINE);
-	    }
+          {
+            vty_out (vty, "    %s filtered by", dist->ifname);
+            has_print = 0;
+            has_print = distribute_print(vty, dist->list,   0,
+                                         DISTRIBUTE_V4_OUT, has_print);
+            has_print = distribute_print(vty, dist->prefix, 1,
+                                         DISTRIBUTE_V4_OUT, has_print);
+            has_print = distribute_print(vty, dist->list,   0,
+                                         DISTRIBUTE_V6_OUT, has_print);
+            has_print = distribute_print(vty, dist->prefix, 1,
+                                         DISTRIBUTE_V6_OUT, has_print);
+            if (has_print)
+              vty_out (vty, "%s", VTY_NEWLINE);
+            else
+              vty_out(vty, " nothing%s", VTY_NEWLINE);
+          }
       }
 
 
   /* Input filter configuration. */
   dist = distribute_lookup (NULL);
-  if (dist && (dist->list[DISTRIBUTE_IN] || dist->prefix[DISTRIBUTE_IN]))
+  vty_out(vty, "  Incoming update filter list for all interface is");
+  has_print = 0;
+  if (dist)
     {
-      vty_out (vty, "  Incoming update filter list for all interface is");
-      if (dist->list[DISTRIBUTE_IN])
-	vty_out (vty, " %s", dist->list[DISTRIBUTE_IN]);
-      if (dist->prefix[DISTRIBUTE_IN])
-	vty_out (vty, "%s (prefix-list) %s",
-		 dist->list[DISTRIBUTE_IN] ? "," : "",
-		 dist->prefix[DISTRIBUTE_IN]);
-      vty_out (vty, "%s", VTY_NEWLINE);
+      has_print = distribute_print(vty, dist->list,   0,
+                                   DISTRIBUTE_V4_IN, has_print);
+      has_print = distribute_print(vty, dist->prefix, 1,
+                                   DISTRIBUTE_V4_IN, has_print);
+      has_print = distribute_print(vty, dist->list,   0,
+                                   DISTRIBUTE_V6_IN, has_print);
+      has_print = distribute_print(vty, dist->prefix, 1,
+                                   DISTRIBUTE_V6_IN, has_print);
     }
+  if (has_print)
+    vty_out (vty, "%s", VTY_NEWLINE);
   else
-    vty_out (vty, "  Incoming update filter list for all interface is not set%s", VTY_NEWLINE);
+    vty_out (vty, " not set%s", VTY_NEWLINE);
 
   for (i = 0; i < disthash->size; i++)
     for (mp = disthash->index[i]; mp; mp = mp->next)
       {
 	dist = mp->data;
-	if (dist->ifname)
-	  if (dist->list[DISTRIBUTE_IN] || dist->prefix[DISTRIBUTE_IN])
-	    {
-	      vty_out (vty, "    %s filtered by", dist->ifname);
-	      if (dist->list[DISTRIBUTE_IN])
-		vty_out (vty, " %s", dist->list[DISTRIBUTE_IN]);
-	      if (dist->prefix[DISTRIBUTE_IN])
-		vty_out (vty, "%s (prefix-list) %s",
-			 dist->list[DISTRIBUTE_IN] ? "," : "",
-			 dist->prefix[DISTRIBUTE_IN]);
-	      vty_out (vty, "%s", VTY_NEWLINE);
-	    }
+        if (dist->ifname)
+          {
+            vty_out (vty, "    %s filtered by", dist->ifname);
+            has_print = 0;
+            has_print = distribute_print(vty, dist->list,   0,
+                                         DISTRIBUTE_V4_IN, has_print);
+            has_print = distribute_print(vty, dist->prefix, 1,
+                                         DISTRIBUTE_V4_IN, has_print);
+            has_print = distribute_print(vty, dist->list,   0,
+                                         DISTRIBUTE_V6_IN, has_print);
+            has_print = distribute_print(vty, dist->prefix, 1,
+                                         DISTRIBUTE_V6_IN, has_print);
+            if (has_print)
+              vty_out (vty, "%s", VTY_NEWLINE);
+            else
+              vty_out(vty, " nothing%s", VTY_NEWLINE);
+          }
       }
   return 0;
 }
@@ -693,6 +928,8 @@ int
 config_write_distribute (struct vty *vty)
 {
   unsigned int i;
+  int j;
+  int output, v6;
   struct hash_backet *mp;
   int write = 0;
 
@@ -703,38 +940,27 @@ config_write_distribute (struct vty *vty)
 
 	dist = mp->data;
 
-	if (dist->list[DISTRIBUTE_IN])
-	  {
-	    vty_out (vty, " distribute-list %s in %s%s", 
-		     dist->list[DISTRIBUTE_IN],
+	for (j=0; j < DISTRIBUTE_MAX; j++)
+	  if (dist->list[j]) {
+	    output = j == DISTRIBUTE_V4_OUT || j == DISTRIBUTE_V6_OUT;
+            v6 = j == DISTRIBUTE_V6_IN || j == DISTRIBUTE_V6_OUT;
+	    vty_out (vty, " %sdistribute-list %s %s %s%s",
+                     v6 ? "ipv6 " : "",
+		     dist->list[j],
+		     output ? "out" : "in",
 		     dist->ifname ? dist->ifname : "",
 		     VTY_NEWLINE);
 	    write++;
 	  }
 
-	if (dist->list[DISTRIBUTE_OUT])
-	  {
-	    vty_out (vty, " distribute-list %s out %s%s", 
-
-		     dist->list[DISTRIBUTE_OUT],
-		     dist->ifname ? dist->ifname : "",
-		     VTY_NEWLINE);
-	    write++;
-	  }
-
-	if (dist->prefix[DISTRIBUTE_IN])
-	  {
-	    vty_out (vty, " distribute-list prefix %s in %s%s",
-		     dist->prefix[DISTRIBUTE_IN],
-		     dist->ifname ? dist->ifname : "",
-		     VTY_NEWLINE);
-	    write++;
-	  }
-
-	if (dist->prefix[DISTRIBUTE_OUT])
-	  {
-	    vty_out (vty, " distribute-list prefix %s out %s%s",
-		     dist->prefix[DISTRIBUTE_OUT],
+	for (j=0; j < DISTRIBUTE_MAX; j++)
+	  if (dist->prefix[j]) {
+	    output = j == DISTRIBUTE_V4_OUT || j == DISTRIBUTE_V6_OUT;
+            v6 = j == DISTRIBUTE_V6_IN || j == DISTRIBUTE_V6_OUT;
+	    vty_out (vty, " %sdistribute-list prefix %s %s %s%s",
+                     v6 ? "ipv6 " : "",
+		     dist->prefix[j],
+		     output ? "out" : "in",
 		     dist->ifname ? dist->ifname : "",
 		     VTY_NEWLINE);
 	    write++;
@@ -756,8 +982,8 @@ distribute_list_init (int node)
 {
   disthash = hash_create (distribute_hash_make,
                           (int (*) (const void *, const void *)) distribute_cmp);
-
-  if(node==RIP_NODE) {
+  /* install v4 */
+  if (node == RIP_NODE || node == BABEL_NODE) {
     install_element (node, &distribute_list_all_cmd);
     install_element (node, &no_distribute_list_all_cmd);
     install_element (node, &distribute_list_cmd);
@@ -766,10 +992,10 @@ distribute_list_init (int node)
     install_element (node, &no_distribute_list_prefix_all_cmd);
     install_element (node, &distribute_list_prefix_cmd);
     install_element (node, &no_distribute_list_prefix_cmd);
-  } else if (node == RIPNG_NODE || node == BABEL_NODE) {
-    /* WARNING: two identical commands installed do a crash, so be worry with
-     aliases. For this reason, and because all these commands are aliases, Babel
-     is not set with RIP. */
+  }
+
+  /* install v6 */
+  if (node == RIPNG_NODE || node == BABEL_NODE) {
     install_element (node, &ipv6_distribute_list_all_cmd);
     install_element (node, &no_ipv6_distribute_list_all_cmd);
     install_element (node, &ipv6_distribute_list_cmd);
@@ -779,4 +1005,16 @@ distribute_list_init (int node)
     install_element (node, &ipv6_distribute_list_prefix_cmd);
     install_element (node, &no_ipv6_distribute_list_prefix_cmd);
   }
+
+  /* install v4 syntax command for v6 only protocols. */
+  if (node == RIPNG_NODE) {
+    install_element (node, &ipv6_as_v4_distribute_list_all_cmd);
+    install_element (node, &no_ipv6_as_v4_distribute_list_all_cmd);
+    install_element (node, &ipv6_as_v4_distribute_list_cmd);
+    install_element (node, &no_ipv6_as_v4_distribute_list_cmd);
+    install_element (node, &ipv6_as_v4_distribute_list_prefix_all_cmd);
+    install_element (node, &no_ipv6_as_v4_distribute_list_prefix_all_cmd);
+    install_element (node, &ipv6_as_v4_distribute_list_prefix_cmd);
+    install_element (node, &no_ipv6_as_v4_distribute_list_prefix_cmd);
+  }
 }
