@@ -1690,6 +1690,8 @@ DEFUN (show_ipv6_ospf6_linkstate,
   struct listnode *node;
   struct ospf6_area *oa;
 
+  OSPF6_CMD_CHECK_RUNNING ();
+
   for (ALL_LIST_ELEMENTS_RO (ospf6->area_list, node, oa))
     {
       vty_out (vty, "%s        SPF Result in Area %s%s%s",
@@ -1738,6 +1740,8 @@ DEFUN (show_ipv6_ospf6_linkstate_detail,
   struct listnode *node;
   struct ospf6_area *oa;
 
+  OSPF6_CMD_CHECK_RUNNING ();
+
   /* copy argv to sargv and then append "detail" */
   for (i = 0; i < argc; i++)
     sargv[i] = argv[i];
@@ -1764,7 +1768,7 @@ ospf6_init (void)
   ospf6_area_init ();
   ospf6_interface_init ();
   ospf6_neighbor_init ();
-  ospf6_zebra_init ();
+  ospf6_zebra_init (master);
 
   ospf6_lsa_init ();
   ospf6_spf_init ();
@@ -1790,22 +1794,17 @@ ospf6_init (void)
   install_element_ospf6_debug_abr ();
   install_element_ospf6_debug_flood ();
 
+  install_element_ospf6_clear_interface ();
+
   install_element (VIEW_NODE, &show_version_ospf6_cmd);
-  install_element (ENABLE_NODE, &show_version_ospf6_cmd);
 
   install_element (VIEW_NODE, &show_ipv6_ospf6_border_routers_cmd);
   install_element (VIEW_NODE, &show_ipv6_ospf6_border_routers_detail_cmd);
-  install_element (ENABLE_NODE, &show_ipv6_ospf6_border_routers_cmd);
-  install_element (ENABLE_NODE, &show_ipv6_ospf6_border_routers_detail_cmd);
 
   install_element (VIEW_NODE, &show_ipv6_ospf6_linkstate_cmd);
   install_element (VIEW_NODE, &show_ipv6_ospf6_linkstate_router_cmd);
   install_element (VIEW_NODE, &show_ipv6_ospf6_linkstate_network_cmd);
   install_element (VIEW_NODE, &show_ipv6_ospf6_linkstate_detail_cmd);
-  install_element (ENABLE_NODE, &show_ipv6_ospf6_linkstate_cmd);
-  install_element (ENABLE_NODE, &show_ipv6_ospf6_linkstate_router_cmd);
-  install_element (ENABLE_NODE, &show_ipv6_ospf6_linkstate_network_cmd);
-  install_element (ENABLE_NODE, &show_ipv6_ospf6_linkstate_detail_cmd);
 
 #define INSTALL(n,c) \
   install_element (n ## _NODE, &show_ipv6_ospf6_ ## c)
@@ -1847,43 +1846,6 @@ ospf6_init (void)
   INSTALL (VIEW, database_type_self_originated_linkstate_id_cmd);
   INSTALL (VIEW, database_type_self_originated_linkstate_id_detail_cmd);
 
-  INSTALL (ENABLE, database_cmd);
-  INSTALL (ENABLE, database_detail_cmd);
-  INSTALL (ENABLE, database_type_cmd);
-  INSTALL (ENABLE, database_type_detail_cmd);
-  INSTALL (ENABLE, database_id_cmd);
-  INSTALL (ENABLE, database_id_detail_cmd);
-  INSTALL (ENABLE, database_linkstate_id_cmd);
-  INSTALL (ENABLE, database_linkstate_id_detail_cmd);
-  INSTALL (ENABLE, database_router_cmd);
-  INSTALL (ENABLE, database_router_detail_cmd);
-  INSTALL (ENABLE, database_adv_router_cmd);
-  INSTALL (ENABLE, database_adv_router_detail_cmd);
-  INSTALL (ENABLE, database_type_id_cmd);
-  INSTALL (ENABLE, database_type_id_detail_cmd);
-  INSTALL (ENABLE, database_type_linkstate_id_cmd);
-  INSTALL (ENABLE, database_type_linkstate_id_detail_cmd);
-  INSTALL (ENABLE, database_type_router_cmd);
-  INSTALL (ENABLE, database_type_router_detail_cmd);
-  INSTALL (ENABLE, database_type_adv_router_cmd);
-  INSTALL (ENABLE, database_type_adv_router_detail_cmd);
-  INSTALL (ENABLE, database_adv_router_linkstate_id_cmd);
-  INSTALL (ENABLE, database_adv_router_linkstate_id_detail_cmd);
-  INSTALL (ENABLE, database_id_router_cmd);
-  INSTALL (ENABLE, database_id_router_detail_cmd);
-  INSTALL (ENABLE, database_type_id_router_cmd);
-  INSTALL (ENABLE, database_type_id_router_detail_cmd);
-  INSTALL (ENABLE, database_type_adv_router_linkstate_id_cmd);
-  INSTALL (ENABLE, database_type_adv_router_linkstate_id_detail_cmd);
-  INSTALL (ENABLE, database_self_originated_cmd);
-  INSTALL (ENABLE, database_self_originated_detail_cmd);
-  INSTALL (ENABLE, database_type_self_originated_cmd);
-  INSTALL (ENABLE, database_type_self_originated_detail_cmd);
-  INSTALL (ENABLE, database_type_id_self_originated_cmd);
-  INSTALL (ENABLE, database_type_id_self_originated_detail_cmd);
-  INSTALL (ENABLE, database_type_self_originated_linkstate_id_cmd);
-  INSTALL (ENABLE, database_type_self_originated_linkstate_id_detail_cmd);
-
   /* Make ospf protocol socket. */
   ospf6_serv_sock ();
   thread_add_read (master, ospf6_receive, NULL, ospf6_sock);
@@ -1892,6 +1854,8 @@ ospf6_init (void)
 void
 ospf6_clean (void)
 {
+  if (!ospf6)
+    return;
   if (ospf6->route_table)
     ospf6_route_remove_all (ospf6->route_table);
   if (ospf6->brouter_table)
