@@ -29,8 +29,11 @@
 #include "connected.h"
 #include "memory.h"
 #include "log.h"
+#include "vrf.h"
+#include "vty.h"
 
 #include "zebra/interface.h"
+#include "zebra/rib.h"
 
 /* Interface looking up using infamous SIOCGIFCONF. */
 static int
@@ -442,8 +445,13 @@ interface_info_ioctl ()
 
 /* Lookup all interface information. */
 void
-interface_list ()
+interface_list (struct zebra_vrf *zvrf)
 {
+  if (zvrf->vrf_id != VRF_DEFAULT)
+    {
+      zlog_warn ("interface_list: ignore VRF %u", zvrf->vrf_id);
+      return;
+    }
   /* Linux can do both proc & ioctl, ioctl is the only way to get
      interface aliases in 2.2 series kernels. */
 #ifdef HAVE_PROC_NET_DEV
