@@ -23,12 +23,11 @@
 #include <zebra.h>
 
 #include "zebra/zserv.h"
-
-extern int interface_lookup_netlink (void);
+#include "rt_netlink.h"
 
 /* Interface information read by netlink. */
 void
-interface_list (void)
+interface_list (struct zebra_vrf *zvrf)
 {
-  interface_lookup_netlink ();  
+  interface_lookup_netlink (zvrf);
 }
